@@ -20,6 +20,7 @@ import (
 	"fmt"
 	"regexp"
 	"strings"
+	"time"
 
 	"github.com/nats-io/nats.go"
 	"github.com/nats-io/nuid"
@@ -46,16 +47,16 @@ type (
 	Publisher interface {
 		// Publish performs a synchronous publish to a stream and waits for ack from server
 		// It accepts subject name (which must be bound to a stream) and message data
-		Publish(context.Context, string, []byte, ...PublishOpt) (*PubAck, error)
+		Publish(ctx context.Context, subject string, payload []byte, opts ...PublishOpt) (*PubAck, error)
 		// PublishMsg performs a synchronous publish to a stream and waits for ack from server
 		// It accepts subject name (which must be bound to a stream) and nats.Message
-		PublishMsg(context.Context, *nats.Msg, ...PublishOpt) (*PubAck, error)
-		// PublishAsync performs a asynchronous publish to a stream and returns [PubAckFuture] interface
+		PublishMsg(ctx context.Context, msg *nats.Msg, opts ...PublishOpt) (*PubAck, error)
+		// PublishAsync performs an asynchronous publish to a stream and returns [PubAckFuture] interface
 		// It accepts subject name (which must be bound to a stream) and message data
-		PublishAsync(string, []byte, ...PublishOpt) (PubAckFuture, error)
-		// PublishMsgAsync performs a asynchronous publish to a stream and returns [PubAckFuture] interface
+		PublishAsync(subject string, payload []byte, opts ...PublishOpt) (PubAckFuture, error)
+		// PublishMsgAsync performs an asynchronous publish to a stream and returns [PubAckFuture] interface
 		// It accepts subject name (which must be bound to a stream) and nats.Message
-		PublishMsgAsync(*nats.Msg, ...PublishOpt) (PubAckFuture, error)
+		PublishMsgAsync(msg *nats.Msg, opts ...PublishOpt) (PubAckFuture, error)
 		// PublishAsyncPending returns the number of async publishes outstanding for this context
 		PublishAsyncPending() int
 		// PublishAsyncComplete returns a channel that will be closed when all outstanding messages are ack'd
@@ -64,37 +65,39 @@ type (
 
 	StreamManager interface {
 		// CreateStream creates a new stream with given config and returns a hook to operate on it
-		CreateStream(context.Context, StreamConfig) (Stream, error)
+		CreateStream(ctx context.Context, cfg StreamConfig) (Stream, error)
 		// UpdateStream updates an existing stream
-		UpdateStream(context.Context, StreamConfig) (Stream, error)
+		UpdateStream(ctx context.Context, cfg StreamConfig) (Stream, error)
 		// Stream returns a [Stream] hook for a given stream name
-		Stream(context.Context, string) (Stream, error)
+		Stream(ctx context.Context, stream string) (Stream, error)
 		// StreamNameBySubject returns a stream name stream listening on given subject
-		StreamNameBySubject(context.Context, string) (string, error)
+		StreamNameBySubject(ctx context.Context, subject string) (string, error)
 		// DeleteStream removes a stream with given name
-		DeleteStream(context.Context, string) error
+		DeleteStream(ctx context.Context, stream string) error
 		// ListStreams returns StreamInfoLister enabling iterating over a channel of stream infos
-		ListStreams(context.Context) StreamInfoLister
+		ListStreams(context.Context, ...StreamListOpt) StreamInfoLister
 		// StreamNames returns a  StreamNameLister enabling iterating over a channel of stream names
-		StreamNames(context.Context) StreamNameLister
+		StreamNames(context.Context, ...StreamListOpt) StreamNameLister
 	}
 
 	StreamConsumerManager interface {
 		// CreateOrUpdateConsumer creates a consumer on a given stream with given config.
 		// If consumer already exists, it will be updated (if possible).
 		// Consumer interface is returned, serving as a hook to operate on a consumer (e.g. fetch messages)
-		CreateOrUpdateConsumer(context.Context, string, ConsumerConfig) (Consumer, error)
+		CreateOrUpdateConsumer(ctx context.Context, stream string, cfg ConsumerConfig) (Consumer, error)
 		// OrderedConsumer returns an OrderedConsumer instance.
 		// OrderedConsumer allows fetching messages from a stream (just like standard consumer),
 		// for in order delivery of messages. Underlying consumer is re-created when necessary,
 		// without additional client code.
-		OrderedConsumer(context.Context, string, OrderedConsumerConfig) (Consumer, error)
+		OrderedConsumer(ctx context.Context, stream string, cfg OrderedConsumerConfig) (Consumer, error)
 		// Consumer returns a hook to an existing consumer, allowing processing of messages
-		Consumer(context.Context, string, string) (Consumer, error)
+		Consumer(ctx context.Context, stream string, consumer string) (Consumer, error)
 		// DeleteConsumer removes a consumer with given name from a stream
-		DeleteConsumer(context.Context, string, string) error
+		DeleteConsumer(ctx context.Context, stream string, consumer string) error
 	}
 
+	StreamListOpt func(*streamsRequest) error
+
 	// AccountInfo contains info about the JetStream usage from the current account.
 	AccountInfo struct {
 		Memory    uint64        `json:"memory"`
@@ -157,12 +160,12 @@ type (
 
 	StreamInfoLister interface {
 		Info() <-chan *StreamInfo
-		Err() <-chan error
+		Err() error
 	}
 
 	StreamNameLister interface {
 		Name() <-chan string
-		Err() <-chan error
+		Err() error
 	}
 
 	apiPagedRequest struct {
@@ -175,7 +178,7 @@ type (
 
 		streams chan *StreamInfo
 		names   chan string
-		errs    chan error
+		err     error
 	}
 
 	streamListResponse struct {
@@ -191,17 +194,21 @@ type (
 	}
 
 	streamsRequest struct {
+		apiPagedRequest
 		Subject string `json:"subject,omitempty"`
 	}
 )
 
+// defaultAPITimeout is used if context.Background() or context.TODO() is passed to API calls.
+const defaultAPITimeout = 5 * time.Second
+
 var subjectRegexp = regexp.MustCompile(`^[^ >]*[>]?$`)
 
-// New returns a enw JetStream instance
+// New returns a new JetStream instance.
 //
 // Available options:
-// [WithClientTrace] - enables request/response tracing
-// [WithPublishAsyncErrHandler] - sets error handler for async message publish
+// [WithClientTrace] - enables request/response tracing.
+// [WithPublishAsyncErrHandler] - sets error handler for async message publish.
 // [WithPublishAsyncMaxPending] - sets the maximum outstanding async publishes that can be inflight at one time.
 // [WithDirectGet] - specifies whether client should use direct get requests.
 func New(nc *nats.Conn, opts ...JetStreamOpt) (JetStream, error) {
@@ -297,6 +304,10 @@ func (js *jetStream) CreateStream(ctx context.Context, cfg StreamConfig) (Stream
 	if err := validateStreamName(cfg.Name); err != nil {
 		return nil, err
 	}
+	ctx, cancel := wrapContextWithoutDeadline(ctx)
+	if cancel != nil {
+		defer cancel()
+	}
 	ncfg := cfg
 	// If we have a mirror and an external domain, convert to ext.APIPrefix.
 	if ncfg.Mirror != nil && ncfg.Mirror.Domain != "" {
@@ -378,6 +389,10 @@ func (js *jetStream) UpdateStream(ctx context.Context, cfg StreamConfig) (Stream
 	if err := validateStreamName(cfg.Name); err != nil {
 		return nil, err
 	}
+	ctx, cancel := wrapContextWithoutDeadline(ctx)
+	if cancel != nil {
+		defer cancel()
+	}
 
 	req, err := json.Marshal(cfg)
 	if err != nil {
@@ -409,6 +424,10 @@ func (js *jetStream) Stream(ctx context.Context, name string) (Stream, error) {
 	if err := validateStreamName(name); err != nil {
 		return nil, err
 	}
+	ctx, cancel := wrapContextWithoutDeadline(ctx)
+	if cancel != nil {
+		defer cancel()
+	}
 	infoSubject := apiSubj(js.apiPrefix, fmt.Sprintf(apiStreamInfoT, name))
 
 	var resp streamInfoResponse
@@ -434,6 +453,10 @@ func (js *jetStream) DeleteStream(ctx context.Context, name string) error {
 	if err := validateStreamName(name); err != nil {
 		return err
 	}
+	ctx, cancel := wrapContextWithoutDeadline(ctx)
+	if cancel != nil {
+		defer cancel()
+	}
 	deleteSubject := apiSubj(js.apiPrefix, fmt.Sprintf(apiStreamDeleteT, name))
 	var resp streamDeleteResponse
 
@@ -473,6 +496,10 @@ func (js *jetStream) OrderedConsumer(ctx context.Context, stream string, cfg Ord
 	if cfg.OptStartSeq != 0 {
 		oc.cursor.streamSeq = cfg.OptStartSeq - 1
 	}
+	err := oc.reset()
+	if err != nil {
+		return nil, err
+	}
 
 	return oc, nil
 }
@@ -514,6 +541,10 @@ func validateSubject(subject string) error {
 }
 
 func (js *jetStream) AccountInfo(ctx context.Context) (*AccountInfo, error) {
+	ctx, cancel := wrapContextWithoutDeadline(ctx)
+	if cancel != nil {
+		defer cancel()
+	}
 	var resp accountInfoResponse
 
 	infoSubject := apiSubj(js.apiPrefix, apiAccountInfo)
@@ -537,29 +568,43 @@ func (js *jetStream) AccountInfo(ctx context.Context) (*AccountInfo, error) {
 }
 
 // ListStreams returns StreamInfoLister enabling iterating over a channel of stream infos
-func (js *jetStream) ListStreams(ctx context.Context) StreamInfoLister {
+//
+// Available options:
+// [WithStreamListSubject] - allows filtering returned streams by provided subject
+func (js *jetStream) ListStreams(ctx context.Context, opts ...StreamListOpt) StreamInfoLister {
 	l := &streamLister{
 		js:      js,
 		streams: make(chan *StreamInfo),
-		errs:    make(chan error, 1),
+	}
+	var streamsReq streamsRequest
+	for _, opt := range opts {
+		if err := opt(&streamsReq); err != nil {
+			l.err = err
+			close(l.streams)
+			return l
+		}
 	}
 	go func() {
+		defer close(l.streams)
+		ctx, cancel := wrapContextWithoutDeadline(ctx)
+		if cancel != nil {
+			defer cancel()
+		}
 		for {
-			page, err := l.streamInfos(ctx)
+			page, err := l.streamInfos(ctx, streamsReq)
 			if err != nil && !errors.Is(err, ErrEndOfData) {
-				l.errs <- err
+				l.err = err
 				return
 			}
 			for _, info := range page {
 				select {
 				case l.streams <- info:
 				case <-ctx.Done():
-					l.errs <- ctx.Err()
+					l.err = ctx.Err()
 					return
 				}
 			}
 			if errors.Is(err, ErrEndOfData) {
-				l.errs <- err
 				return
 			}
 		}
@@ -574,34 +619,48 @@ func (s *streamLister) Info() <-chan *StreamInfo {
 }
 
 // Err returns an error channel which will be populated with error from [ListStreams] or [StreamNames] request
-func (s *streamLister) Err() <-chan error {
-	return s.errs
+func (s *streamLister) Err() error {
+	return s.err
 }
 
 // StreamNames returns a [StreamNameLister] enabling iterating over a channel of stream names
-func (js *jetStream) StreamNames(ctx context.Context) StreamNameLister {
+//
+// Available options:
+// [WithStreamListSubject] - allows filtering returned streams by provided subject
+func (js *jetStream) StreamNames(ctx context.Context, opts ...StreamListOpt) StreamNameLister {
 	l := &streamLister{
 		js:    js,
 		names: make(chan string),
-		errs:  make(chan error, 1),
+	}
+	var streamsReq streamsRequest
+	for _, opt := range opts {
+		if err := opt(&streamsReq); err != nil {
+			l.err = err
+			close(l.streams)
+			return l
+		}
 	}
 	go func() {
+		ctx, cancel := wrapContextWithoutDeadline(ctx)
+		if cancel != nil {
+			defer cancel()
+		}
+		defer close(l.names)
 		for {
-			page, err := l.streamNames(ctx)
+			page, err := l.streamNames(ctx, streamsReq)
 			if err != nil && !errors.Is(err, ErrEndOfData) {
-				l.errs <- err
+				l.err = err
 				return
 			}
 			for _, info := range page {
 				select {
 				case l.names <- info:
 				case <-ctx.Done():
-					l.errs <- ctx.Err()
+					l.err = ctx.Err()
 					return
 				}
 			}
 			if errors.Is(err, ErrEndOfData) {
-				l.errs <- err
 				return
 			}
 		}
@@ -611,6 +670,10 @@ func (js *jetStream) StreamNames(ctx context.Context) StreamNameLister {
 }
 
 func (js *jetStream) StreamNameBySubject(ctx context.Context, subject string) (string, error) {
+	ctx, cancel := wrapContextWithoutDeadline(ctx)
+	if cancel != nil {
+		defer cancel()
+	}
 	if err := validateSubject(subject); err != nil {
 		return "", err
 	}
@@ -642,21 +705,28 @@ func (s *streamLister) Name() <-chan string {
 }
 
 // infos fetches the next [StreamInfo] page
-func (s *streamLister) streamInfos(ctx context.Context) ([]*StreamInfo, error) {
+func (s *streamLister) streamInfos(ctx context.Context, streamsReq streamsRequest) ([]*StreamInfo, error) {
 	if s.pageInfo != nil && s.offset >= s.pageInfo.Total {
 		return nil, ErrEndOfData
 	}
 
-	req, err := json.Marshal(
-		apiPagedRequest{Offset: s.offset},
-	)
+	req := streamsRequest{
+		apiPagedRequest: apiPagedRequest{
+			Offset: s.offset,
+		},
+		Subject: streamsReq.Subject,
+	}
+	reqJSON, err := json.Marshal(req)
+	if err != nil {
+		return nil, err
+	}
 	if err != nil {
 		return nil, err
 	}
 
 	slSubj := apiSubj(s.js.apiPrefix, apiStreamListT)
 	var resp streamListResponse
-	_, err = s.js.apiRequestJSON(ctx, slSubj, &resp, req)
+	_, err = s.js.apiRequestJSON(ctx, slSubj, &resp, reqJSON)
 	if err != nil {
 		return nil, err
 	}
@@ -670,21 +740,25 @@ func (s *streamLister) streamInfos(ctx context.Context) ([]*StreamInfo, error) {
 }
 
 // streamNames fetches the next stream names page
-func (s *streamLister) streamNames(ctx context.Context) ([]string, error) {
+func (s *streamLister) streamNames(ctx context.Context, streamsReq streamsRequest) ([]string, error) {
 	if s.pageInfo != nil && s.offset >= s.pageInfo.Total {
 		return nil, ErrEndOfData
 	}
 
-	req, err := json.Marshal(
-		apiPagedRequest{Offset: s.offset},
-	)
+	req := streamsRequest{
+		apiPagedRequest: apiPagedRequest{
+			Offset: s.offset,
+		},
+		Subject: streamsReq.Subject,
+	}
+	reqJSON, err := json.Marshal(req)
 	if err != nil {
 		return nil, err
 	}
 
 	slSubj := apiSubj(s.js.apiPrefix, apiStreams)
 	var resp streamNamesResponse
-	_, err = s.js.apiRequestJSON(ctx, slSubj, &resp, req)
+	_, err = s.js.apiRequestJSON(ctx, slSubj, &resp, reqJSON)
 	if err != nil {
 		return nil, err
 	}
@@ -696,3 +770,13 @@ func (s *streamLister) streamNames(ctx context.Context) ([]string, error) {
 	s.offset += len(resp.Streams)
 	return resp.Streams, nil
 }
+
+// wrapContextWithoutDeadline wraps context without deadline with default timeout.
+// If deadline is already set, it will be returned as is, and cancel() will be nil.
+// Caller should check if cancel() is nil before calling it.
+func wrapContextWithoutDeadline(ctx context.Context) (context.Context, context.CancelFunc) {
+	if _, ok := ctx.Deadline(); ok {
+		return ctx, nil
+	}
+	return context.WithTimeout(ctx, defaultAPITimeout)
+}
