@@ -18,6 +18,7 @@ import (
 	"errors"
 	"fmt"
 	"os"
+	"reflect"
 	"testing"
 	"time"
 
@@ -204,31 +205,48 @@ func TestCreateStream(t *testing.T) {
 		name      string
 		stream    string
 		subject   string
+		timeout   time.Duration
 		withError error
 	}{
 		{
 			name:    "create stream, ok",
 			stream:  "foo",
+			timeout: 10 * time.Second,
+			subject: "FOO.123",
+		},
+		{
+			name:    "with empty context",
+			stream:  "foo",
 			subject: "FOO.123",
 		},
 		{
 			name:      "invalid stream name",
 			stream:    "foo.123",
 			subject:   "FOO.123",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrInvalidStreamName,
 		},
 		{
 			name:      "stream name required",
 			stream:    "",
 			subject:   "FOO.123",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrStreamNameRequired,
 		},
 		{
 			name:      "stream name already in use",
 			stream:    "foo",
 			subject:   "BAR.123",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrStreamNameAlreadyInUse,
 		},
+		{
+			name:      "context timeout",
+			stream:    "foo",
+			subject:   "BAR.123",
+			timeout:   1 * time.Microsecond,
+			withError: context.DeadlineExceeded,
+		},
 	}
 
 	srv := RunBasicJetStreamServer()
@@ -238,8 +256,6 @@ func TestCreateStream(t *testing.T) {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 
-	ctx, cancel := context.WithTimeout(context.Background(), 20*time.Second)
-	defer cancel()
 	js, err := jetstream.New(nc)
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
@@ -248,6 +264,12 @@ func TestCreateStream(t *testing.T) {
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			ctx := context.Background()
+			if test.timeout > 0 {
+				var cancel context.CancelFunc
+				ctx, cancel = context.WithTimeout(context.Background(), test.timeout)
+				defer cancel()
+			}
 			_, err = js.CreateStream(ctx, jetstream.StreamConfig{Name: test.stream, Subjects: []string{test.subject}})
 			if test.withError != nil {
 				if !errors.Is(err, test.withError) {
@@ -262,36 +284,200 @@ func TestCreateStream(t *testing.T) {
 	}
 }
 
+func TestCreateStreamMirrorCrossDomains(t *testing.T) {
+	test := []struct {
+		name         string
+		streamConfig *jetstream.StreamConfig
+	}{
+		{
+			name: "create stream mirror cross domains",
+			streamConfig: &jetstream.StreamConfig{
+				Name: "MIRROR",
+				Mirror: &jetstream.StreamSource{
+					Name:   "TEST",
+					Domain: "HUB",
+				},
+			},
+		},
+		{
+			name: "create stream with source cross domains",
+			streamConfig: &jetstream.StreamConfig{
+				Name: "MIRROR",
+				Sources: []*jetstream.StreamSource{
+					{
+						Name:   "TEST",
+						Domain: "HUB",
+					},
+				},
+			},
+		},
+	}
+
+	for _, test := range test {
+		t.Run(test.name, func(t *testing.T) {
+			conf := createConfFile(t, []byte(`
+		server_name: HUB
+		listen: 127.0.0.1:-1
+		jetstream: { domain: HUB }
+		leafnodes { listen: 127.0.0.1:7422 }
+	}`))
+			defer os.Remove(conf)
+			srv, _ := RunServerWithConfig(conf)
+			defer shutdownJSServerAndRemoveStorage(t, srv)
+
+			lconf := createConfFile(t, []byte(`
+	server_name: LEAF
+	listen: 127.0.0.1:-1
+	 jetstream: { domain:LEAF }
+	 leafnodes {
+		  remotes = [ { url: "leaf://127.0.0.1" } ]
+	 }
+}`))
+			defer os.Remove(lconf)
+			ln, _ := RunServerWithConfig(lconf)
+			defer shutdownJSServerAndRemoveStorage(t, ln)
+
+			nc, err := nats.Connect(srv.ClientURL())
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+			defer nc.Close()
+
+			ctx, cancel := context.WithTimeout(context.Background(), 20*time.Second)
+			defer cancel()
+			js, err := jetstream.New(nc)
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+
+			_, err = js.CreateStream(ctx, jetstream.StreamConfig{
+				Name:     "TEST",
+				Subjects: []string{"foo"},
+			})
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+			if _, err := js.Publish(ctx, "foo", []byte("msg1")); err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+			if _, err := js.Publish(ctx, "foo", []byte("msg2")); err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+
+			lnc, err := nats.Connect(ln.ClientURL())
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+			defer lnc.Close()
+			ljs, err := jetstream.New(lnc)
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+
+			ccfg := *test.streamConfig
+			_, err = ljs.CreateStream(ctx, ccfg)
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+			if !reflect.DeepEqual(test.streamConfig, &ccfg) {
+				t.Fatalf("Did not expect config to be altered: %+v vs %+v", test.streamConfig, ccfg)
+			}
+
+			// Make sure we sync.
+			checkFor(t, 2*time.Second, 15*time.Millisecond, func() error {
+				lStream, err := ljs.Stream(ctx, "MIRROR")
+				if err != nil {
+					t.Fatalf("Unexpected error: %v", err)
+				}
+
+				if lStream.CachedInfo().State.Msgs == 2 {
+					return nil
+				}
+				return fmt.Errorf("Did not get synced messages: %d", lStream.CachedInfo().State.Msgs)
+			})
+			if _, err := ljs.Publish(ctx, "foo", []byte("msg3")); err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+			lStream, err := ljs.Stream(ctx, "MIRROR")
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+
+			if lStream.CachedInfo().State.Msgs != 3 {
+				t.Fatalf("Expected 3 msgs in stream; got: %d", lStream.CachedInfo().State.Msgs)
+			}
+
+			rjs, err := jetstream.NewWithDomain(lnc, "HUB")
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+
+			_, err = rjs.Stream(ctx, "TEST")
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+			if _, err := rjs.Publish(ctx, "foo", []byte("msg4")); err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+			rStream, err := rjs.Stream(ctx, "TEST")
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+
+			if rStream.CachedInfo().State.Msgs != 4 {
+				t.Fatalf("Expected 3 msgs in stream; got: %d", rStream.CachedInfo().State.Msgs)
+			}
+		})
+	}
+}
+
 func TestUpdateStream(t *testing.T) {
 	tests := []struct {
 		name      string
 		stream    string
 		subject   string
+		timeout   time.Duration
 		withError error
 	}{
 		{
 			name:    "update existing stream",
 			stream:  "foo",
 			subject: "BAR.123",
+			timeout: 10 * time.Second,
+		},
+		{
+			name:    "with empty context",
+			stream:  "foo",
+			subject: "FOO.123",
 		},
 		{
 			name:      "invalid stream name",
 			stream:    "foo.123",
 			subject:   "FOO.123",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrInvalidStreamName,
 		},
 		{
 			name:      "stream name required",
 			stream:    "",
 			subject:   "FOO.123",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrStreamNameRequired,
 		},
 		{
 			name:      "stream not found",
 			stream:    "bar",
 			subject:   "FOO.123",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrStreamNotFound,
 		},
+		{
+			name:      "context timeout",
+			stream:    "foo",
+			subject:   "FOO.123",
+			timeout:   1 * time.Microsecond,
+			withError: context.DeadlineExceeded,
+		},
 	}
 
 	srv := RunBasicJetStreamServer()
@@ -301,19 +487,23 @@ func TestUpdateStream(t *testing.T) {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 
-	ctx, cancel := context.WithTimeout(context.Background(), 20*time.Second)
-	defer cancel()
 	js, err := jetstream.New(nc)
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 	defer nc.Close()
-	_, err = js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.123"}})
+	_, err = js.CreateStream(context.Background(), jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.123"}})
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			ctx := context.Background()
+			if test.timeout > 0 {
+				var cancel context.CancelFunc
+				ctx, cancel = context.WithTimeout(context.Background(), test.timeout)
+				defer cancel()
+			}
 			s, err := js.UpdateStream(ctx, jetstream.StreamConfig{Name: test.stream, Subjects: []string{test.subject}})
 			if test.withError != nil {
 				if !errors.Is(err, test.withError) {
@@ -340,27 +530,42 @@ func TestStream(t *testing.T) {
 		name      string
 		stream    string
 		subject   string
+		timeout   time.Duration
 		withError error
 	}{
 		{
-			name:   "get existing stream",
+			name:    "get existing stream",
+			stream:  "foo",
+			timeout: 10 * time.Second,
+		},
+		{
+			name:   "with empty context",
 			stream: "foo",
 		},
 		{
 			name:      "invalid stream name",
 			stream:    "foo.123",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrInvalidStreamName,
 		},
 		{
 			name:      "stream name required",
 			stream:    "",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrStreamNameRequired,
 		},
 		{
 			name:      "stream not found",
 			stream:    "bar",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrStreamNotFound,
 		},
+		{
+			name:      "context timeout",
+			stream:    "foo",
+			timeout:   1 * time.Microsecond,
+			withError: context.DeadlineExceeded,
+		},
 	}
 	srv := RunBasicJetStreamServer()
 	defer shutdownJSServerAndRemoveStorage(t, srv)
@@ -369,20 +574,24 @@ func TestStream(t *testing.T) {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 
-	ctx, cancel := context.WithTimeout(context.Background(), 20*time.Second)
-	defer cancel()
 	js, err := jetstream.New(nc)
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 	defer nc.Close()
-	_, err = js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.123"}})
+	_, err = js.CreateStream(context.Background(), jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.123"}})
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			ctx := context.Background()
+			if test.timeout > 0 {
+				var cancel context.CancelFunc
+				ctx, cancel = context.WithTimeout(context.Background(), test.timeout)
+				defer cancel()
+			}
 			s, err := js.Stream(ctx, test.stream)
 			if test.withError != nil {
 				if !errors.Is(err, test.withError) {
@@ -405,25 +614,34 @@ func TestDeleteStream(t *testing.T) {
 		name      string
 		stream    string
 		subject   string
+		timeout   time.Duration
 		withError error
 	}{
 		{
-			name:   "delete existing stream",
-			stream: "foo",
+			name:    "delete existing stream",
+			stream:  "foo",
+			timeout: 10 * time.Second,
+		},
+		{
+			name:   "with empty context",
+			stream: "bar",
 		},
 		{
 			name:      "invalid stream name",
 			stream:    "foo.123",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrInvalidStreamName,
 		},
 		{
 			name:      "stream name required",
 			stream:    "",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrStreamNameRequired,
 		},
 		{
 			name:      "stream not found",
 			stream:    "foo",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrStreamNotFound,
 		},
 	}
@@ -434,20 +652,29 @@ func TestDeleteStream(t *testing.T) {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 
-	ctx, cancel := context.WithTimeout(context.Background(), 20*time.Second)
-	defer cancel()
 	js, err := jetstream.New(nc)
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 	defer nc.Close()
-	_, err = js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.123"}})
+	_, err = js.CreateStream(context.Background(), jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.123"}})
+	if err != nil {
+		t.Fatalf("Unexpected error: %v", err)
+	}
+	defer nc.Close()
+	_, err = js.CreateStream(context.Background(), jetstream.StreamConfig{Name: "bar", Subjects: []string{"BAR.123"}})
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			ctx := context.Background()
+			if test.timeout > 0 {
+				var cancel context.CancelFunc
+				ctx, cancel = context.WithTimeout(context.Background(), 20*time.Second)
+				defer cancel()
+			}
 			err := js.DeleteStream(ctx, test.stream)
 			if test.withError != nil {
 				if !errors.Is(err, test.withError) {
@@ -552,15 +779,45 @@ func TestListStreams(t *testing.T) {
 	tests := []struct {
 		name       string
 		streamsNum int
+		timeout    time.Duration
+		subject    string
+		expected   int
 		withError  error
 	}{
 		{
 			name:       "list streams",
 			streamsNum: 260,
+			timeout:    10 * time.Second,
+			expected:   260,
+		},
+		{
+			name:       "with empty context",
+			streamsNum: 260,
+			expected:   260,
 		},
 		{
 			name:       "no stream available",
+			timeout:    10 * time.Second,
 			streamsNum: 0,
+			expected:   0,
+		},
+		{
+			name:       "list streams with subject filter",
+			subject:    "FOO.123",
+			streamsNum: 260,
+			expected:   1,
+		},
+		{
+			name:       "list streams with subject filter, no match",
+			subject:    "FOO.500",
+			streamsNum: 100,
+			expected:   0,
+		},
+		{
+			name:       "context timeout",
+			streamsNum: 260,
+			timeout:    1 * time.Microsecond,
+			withError:  context.DeadlineExceeded,
 		},
 	}
 
@@ -573,34 +830,42 @@ func TestListStreams(t *testing.T) {
 				t.Fatalf("Unexpected error: %v", err)
 			}
 
-			ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
-			defer cancel()
+			ctx := context.Background()
+			if test.timeout > 0 {
+				var cancel context.CancelFunc
+				ctx, cancel = context.WithTimeout(context.Background(), test.timeout)
+				defer cancel()
+			}
 			js, err := jetstream.New(nc)
 			if err != nil {
 				t.Fatalf("Unexpected error: %v", err)
 			}
 			defer nc.Close()
 			for i := 0; i < test.streamsNum; i++ {
-				_, err = js.CreateStream(ctx, jetstream.StreamConfig{Name: fmt.Sprintf("foo%d", i), Subjects: []string{fmt.Sprintf("FOO.%d", i)}})
+				_, err = js.CreateStream(context.Background(), jetstream.StreamConfig{Name: fmt.Sprintf("foo%d", i), Subjects: []string{fmt.Sprintf("FOO.%d", i)}})
 				if err != nil {
 					t.Fatalf("Unexpected error: %v", err)
 				}
 			}
-			streamsList := js.ListStreams(ctx)
-			streams := make([]*jetstream.StreamInfo, 0)
-		Loop:
-			for {
-				select {
-				case s := <-streamsList.Info():
-					streams = append(streams, s)
-				case err := <-streamsList.Err():
-					if !errors.Is(err, jetstream.ErrEndOfData) {
-						t.Fatalf("Unexpected error: %v", err)
-					}
-					break Loop
-				}
+			opts := []jetstream.StreamListOpt{}
+			if test.subject != "" {
+				opts = append(opts, jetstream.WithStreamListSubject(test.subject))
 			}
-			if len(streams) != test.streamsNum {
+			streamsList := js.ListStreams(ctx, opts...)
+			streams := make([]*jetstream.StreamInfo, 0)
+			for si := range streamsList.Info() {
+				streams = append(streams, si)
+			}
+			if test.withError != nil {
+				if !errors.Is(streamsList.Err(), test.withError) {
+					t.Fatalf("Expected error: %v; got: %v", test.withError, streamsList.Err())
+				}
+				return
+			}
+			if streamsList.Err() != nil {
+				t.Fatalf("Unexpected error: %s", streamsList.Err())
+			}
+			if len(streams) != test.expected {
 				t.Fatalf("Wrong number of streams; want: %d; got: %d", test.streamsNum, len(streams))
 			}
 		})
@@ -611,15 +876,46 @@ func TestStreamNames(t *testing.T) {
 	tests := []struct {
 		name       string
 		streamsNum int
+		subject    string
+		expected   int
+		timeout    time.Duration
 		withError  error
 	}{
 		{
 			name:       "list streams",
 			streamsNum: 500,
+			timeout:    10 * time.Second,
+			expected:   500,
+		},
+		{
+			name:       "with empty context",
+			streamsNum: 500,
+			expected:   500,
 		},
 		{
 			name:       "no stream available",
 			streamsNum: 0,
+			expected:   0,
+			timeout:    10 * time.Second,
+		},
+		{
+			name:       "list streams with subject filter",
+			subject:    "FOO.123",
+			streamsNum: 260,
+			expected:   1,
+		},
+		{
+			name:       "list streams with subject filter, no match",
+			subject:    "FOO.500",
+			streamsNum: 100,
+			expected:   0,
+			timeout:    10 * time.Second,
+		},
+		{
+			name:       "context timeout",
+			streamsNum: 500,
+			timeout:    1 * time.Microsecond,
+			withError:  context.DeadlineExceeded,
 		},
 	}
 
@@ -632,34 +928,42 @@ func TestStreamNames(t *testing.T) {
 				t.Fatalf("Unexpected error: %v", err)
 			}
 
-			ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
-			defer cancel()
+			ctx := context.Background()
+			if test.timeout > 0 {
+				var cancel context.CancelFunc
+				ctx, cancel = context.WithTimeout(ctx, test.timeout)
+				defer cancel()
+			}
 			js, err := jetstream.New(nc)
 			if err != nil {
 				t.Fatalf("Unexpected error: %v", err)
 			}
 			defer nc.Close()
 			for i := 0; i < test.streamsNum; i++ {
-				_, err = js.CreateStream(ctx, jetstream.StreamConfig{Name: fmt.Sprintf("foo%d", i), Subjects: []string{fmt.Sprintf("FOO.%d", i)}})
+				_, err = js.CreateStream(context.Background(), jetstream.StreamConfig{Name: fmt.Sprintf("foo%d", i), Subjects: []string{fmt.Sprintf("FOO.%d", i)}})
 				if err != nil {
 					t.Fatalf("Unexpected error: %v", err)
 				}
 			}
-			streamsList := js.StreamNames(ctx)
-			streams := make([]string, 0)
-		Loop:
-			for {
-				select {
-				case s := <-streamsList.Name():
-					streams = append(streams, s)
-				case err := <-streamsList.Err():
-					if !errors.Is(err, jetstream.ErrEndOfData) {
-						t.Fatalf("Unexpected error: %v", err)
-					}
-					break Loop
-				}
+			opts := []jetstream.StreamListOpt{}
+			if test.subject != "" {
+				opts = append(opts, jetstream.WithStreamListSubject(test.subject))
 			}
-			if len(streams) != test.streamsNum {
+			streamsList := js.StreamNames(ctx, opts...)
+			streams := make([]string, 0)
+			for s := range streamsList.Name() {
+				streams = append(streams, s)
+			}
+			if test.withError != nil {
+				if !errors.Is(streamsList.Err(), test.withError) {
+					t.Fatalf("Expected error: %v; got: %v", test.withError, streamsList.Err())
+				}
+				return
+			}
+			if streamsList.Err() != nil {
+				t.Fatalf("Unexpected error: %s", streamsList.Err())
+			}
+			if len(streams) != test.expected {
 				t.Fatalf("Wrong number of streams; want: %d; got: %d", test.streamsNum, len(streams))
 			}
 		})
@@ -672,18 +976,27 @@ func TestJetStream_CreateOrUpdateConsumer(t *testing.T) {
 		stream         string
 		consumerConfig jetstream.ConsumerConfig
 		shouldCreate   bool
+		timeout        time.Duration
 		withError      error
 	}{
 		{
 			name:           "create durable pull consumer",
 			stream:         "foo",
 			consumerConfig: jetstream.ConsumerConfig{Durable: "dur", AckPolicy: jetstream.AckExplicitPolicy},
+			timeout:        10 * time.Second,
 			shouldCreate:   true,
 		},
 		{
 			name:           "create ephemeral pull consumer",
 			stream:         "foo",
 			consumerConfig: jetstream.ConsumerConfig{AckPolicy: jetstream.AckExplicitPolicy},
+			timeout:        10 * time.Second,
+			shouldCreate:   true,
+		},
+		{
+			name:           "with empty context",
+			consumerConfig: jetstream.ConsumerConfig{AckPolicy: jetstream.AckExplicitPolicy},
+			stream:         "foo",
 			shouldCreate:   true,
 		},
 		{
@@ -713,6 +1026,13 @@ func TestJetStream_CreateOrUpdateConsumer(t *testing.T) {
 			consumerConfig: jetstream.ConsumerConfig{Durable: "dur.123", AckPolicy: jetstream.AckExplicitPolicy},
 			withError:      jetstream.ErrInvalidConsumerName,
 		},
+		{
+			name:           "context timeout",
+			consumerConfig: jetstream.ConsumerConfig{AckPolicy: jetstream.AckExplicitPolicy},
+			stream:         "foo",
+			timeout:        1 * time.Microsecond,
+			withError:      context.DeadlineExceeded,
+		},
 	}
 
 	srv := RunBasicJetStreamServer()
@@ -728,15 +1048,19 @@ func TestJetStream_CreateOrUpdateConsumer(t *testing.T) {
 	}
 	defer nc.Close()
 
-	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
-	defer cancel()
-	_, err = js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
+	_, err = js.CreateStream(context.Background(), jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			ctx := context.Background()
+			if test.timeout > 0 {
+				var cancel context.CancelFunc
+				ctx, cancel = context.WithTimeout(context.Background(), test.timeout)
+				defer cancel()
+			}
 			var sub *nats.Subscription
 			if test.consumerConfig.FilterSubject != "" {
 				sub, err = nc.SubscribeSync(fmt.Sprintf("$JS.API.CONSUMER.CREATE.foo.*.%s", test.consumerConfig.FilterSubject))
@@ -771,17 +1095,25 @@ func TestJetStream_Consumer(t *testing.T) {
 		name      string
 		stream    string
 		durable   string
+		timeout   time.Duration
 		withError error
 	}{
 		{
 			name:    "get existing consumer",
 			stream:  "foo",
 			durable: "dur",
+			timeout: 10 * time.Second,
+		},
+		{
+			name:    "with empty context",
+			stream:  "foo",
+			durable: "dur",
 		},
 		{
 			name:      "consumer does not exist",
 			stream:    "foo",
 			durable:   "abc",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrConsumerNotFound,
 		},
 		{
@@ -802,6 +1134,13 @@ func TestJetStream_Consumer(t *testing.T) {
 			durable:   "dur",
 			withError: jetstream.ErrInvalidStreamName,
 		},
+		{
+			name:      "context timeout",
+			stream:    "foo",
+			durable:   "dur",
+			timeout:   1 * time.Microsecond,
+			withError: context.DeadlineExceeded,
+		},
 	}
 
 	srv := RunBasicJetStreamServer()
@@ -817,19 +1156,23 @@ func TestJetStream_Consumer(t *testing.T) {
 	}
 	defer nc.Close()
 
-	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
-	defer cancel()
-	s, err := js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
+	s, err := js.CreateStream(context.Background(), jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
-	_, err = s.CreateOrUpdateConsumer(ctx, jetstream.ConsumerConfig{Durable: "dur", AckPolicy: jetstream.AckAllPolicy, Description: "desc"})
+	_, err = s.CreateOrUpdateConsumer(context.Background(), jetstream.ConsumerConfig{Durable: "dur", AckPolicy: jetstream.AckAllPolicy, Description: "desc"})
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			ctx := context.Background()
+			if test.timeout > 0 {
+				var cancel context.CancelFunc
+				ctx, cancel = context.WithTimeout(context.Background(), test.timeout)
+				defer cancel()
+			}
 			c, err := js.Consumer(ctx, test.stream, test.durable)
 			if test.withError != nil {
 				if err == nil || !errors.Is(err, test.withError) {
@@ -852,17 +1195,25 @@ func TestJetStream_DeleteConsumer(t *testing.T) {
 		name      string
 		stream    string
 		durable   string
+		timeout   time.Duration
 		withError error
 	}{
 		{
 			name:    "delete existing consumer",
 			stream:  "foo",
 			durable: "dur",
+			timeout: 10 * time.Second,
+		},
+		{
+			name:    "with empty context",
+			stream:  "foo",
+			durable: "dur2",
 		},
 		{
 			name:      "consumer does not exist",
 			stream:    "foo",
 			durable:   "dur",
+			timeout:   10 * time.Second,
 			withError: jetstream.ErrConsumerNotFound,
 		},
 		{
@@ -883,6 +1234,13 @@ func TestJetStream_DeleteConsumer(t *testing.T) {
 			durable:   "dur",
 			withError: jetstream.ErrInvalidStreamName,
 		},
+		{
+			name:      "context timeout",
+			stream:    "foo",
+			durable:   "dur",
+			timeout:   1 * time.Microsecond,
+			withError: context.DeadlineExceeded,
+		},
 	}
 
 	srv := RunBasicJetStreamServer()
@@ -898,19 +1256,27 @@ func TestJetStream_DeleteConsumer(t *testing.T) {
 	}
 	defer nc.Close()
 
-	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
-	defer cancel()
-	s, err := js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
+	s, err := js.CreateStream(context.Background(), jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
-	_, err = s.CreateOrUpdateConsumer(ctx, jetstream.ConsumerConfig{Durable: "dur", AckPolicy: jetstream.AckAllPolicy, Description: "desc"})
+	_, err = s.CreateOrUpdateConsumer(context.Background(), jetstream.ConsumerConfig{Durable: "dur", AckPolicy: jetstream.AckAllPolicy, Description: "desc"})
+	if err != nil {
+		t.Fatalf("Unexpected error: %v", err)
+	}
+	_, err = s.CreateOrUpdateConsumer(context.Background(), jetstream.ConsumerConfig{Durable: "dur2", AckPolicy: jetstream.AckAllPolicy, Description: "desc"})
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			ctx := context.Background()
+			if test.timeout > 0 {
+				var cancel context.CancelFunc
+				ctx, cancel = context.WithTimeout(context.Background(), test.timeout)
+				defer cancel()
+			}
 			err := js.DeleteConsumer(ctx, test.stream, test.durable)
 			if test.withError != nil {
 				if err == nil || !errors.Is(err, test.withError) {
@@ -934,11 +1300,18 @@ func TestStreamNameBySubject(t *testing.T) {
 		name      string
 		subject   string
 		withError error
+		timeout   time.Duration
 		expected  string
 	}{
 		{
 			name:     "get stream name by subject explicit",
 			subject:  "FOO.123",
+			timeout:  10 * time.Second,
+			expected: "foo",
+		},
+		{
+			name:     "with empty context",
+			subject:  "FOO.123",
 			expected: "foo",
 		},
 		{
@@ -961,6 +1334,12 @@ func TestStreamNameBySubject(t *testing.T) {
 			subject:   "FOO.>.123",
 			withError: jetstream.ErrInvalidSubject,
 		},
+		{
+			name:      "context timeout",
+			subject:   "FOO.123",
+			timeout:   1 * time.Microsecond,
+			withError: context.DeadlineExceeded,
+		},
 	}
 
 	srv := RunBasicJetStreamServer()
@@ -976,19 +1355,23 @@ func TestStreamNameBySubject(t *testing.T) {
 	}
 	defer nc.Close()
 
-	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
-	defer cancel()
-	_, err = js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
+	_, err = js.CreateStream(context.Background(), jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
-	_, err = js.CreateStream(ctx, jetstream.StreamConfig{Name: "bar", Subjects: []string{"BAR.ABC"}})
+	_, err = js.CreateStream(context.Background(), jetstream.StreamConfig{Name: "bar", Subjects: []string{"BAR.ABC"}})
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			ctx := context.Background()
+			if test.timeout > 0 {
+				var cancel context.CancelFunc
+				ctx, cancel = context.WithTimeout(context.Background(), test.timeout)
+				defer cancel()
+			}
 			name, err := js.StreamNameBySubject(ctx, test.subject)
 			if test.withError != nil {
 				if err == nil || !errors.Is(err, test.withError) {
@@ -1002,7 +1385,6 @@ func TestStreamNameBySubject(t *testing.T) {
 			if test.expected != "" && name != test.expected {
 				t.Fatalf("Unexpected stream name; want: %s; got: %s", test.expected, name)
 			}
-
 		})
 	}
 }
